# -*- coding: future_fstrings -*-

from .dataset import Dataset
from .report import Report


class Import:
    # json keys
    id_key = 'id'
    name_key = 'name'
    created_timedate_key = 'createdDateTime'
    datasets_key = 'datasets'
    import_state_key = 'importState'
    reports_key = 'reports'
    updated_datetime_key = 'updatedDateTime'
    source_key = 'source'
    connection_type_key = 'connectionType'
    value_key = 'value'

    # import state values
    import_state_succeeded = 'Succeeded'
    import_state_publishing = 'Publishing'

    def __init__(self, import_id, name=None, created_datetime=None, datasets=None, import_state=None,
                 reports=None, updated_datetime=None, source=None, connection_type=None):
        self.id = import_id
        self.name = name
        self.created_datetime = created_datetime
        self.datasets = datasets
        self.import_state = import_state
        self.reports = reports
        self.updated_datetime = updated_datetime
        self.source = source
        self.connection_type = connection_type

    @classmethod
    def from_dict(cls, dictionary):
        import_id = dictionary.get(cls.id_key)
        if import_id is None:
            raise RuntimeError("Import dictionary has no id key")

        name = dictionary.get(cls.name_key)
        created_datetime = dictionary.get(cls.created_timedate_key)

        if cls.datasets_key in dictionary:
            datasets = [Dataset.from_dict(x) for x in dictionary.get(cls.datasets_key)]
        else:
            datasets = None

        import_state = dictionary.get(cls.import_state_key)

        if cls.reports_key in dictionary:
            reports = [Report.from_dict(x) for x in dictionary.get(cls.reports_key)]
        else:
            reports = None

        updated_datetime = dictionary.get(cls.updated_datetime_key)
        source = dictionary.get(cls.source_key)
        connection_type = dictionary.get(cls.connection_type_key)

        return cls(import_id, name, created_datetime, datasets, import_state,
                   reports, updated_datetime, source, connection_type)



# -*- coding: future_fstrings -*-
import requests
import json

from requests.exceptions import HTTPError
from .dataset import *


class Datasets:
    # url snippets
    groups_snippet = 'groups'
    datasets_snippet = 'datasets'
    tables_snippet = 'tables'
    rows_snippet = 'rows'
    parameters_snippet = 'parameters'

    # json keys
    get_datasets_value_key = 'value'

    def __init__(self, client):
        self.client = client
        self.base_url = f'{self.client.api_url}/{self.client.api_version_snippet}/{self.client.api_myorg_snippet}'

    def count(self, group_id=None):
        """
        Evaluates the number of datasets
        :param group_id: The optional group id
        :return: The number of datasets as returned by the API
        """
        return len(self.get_datasets(group_id))

    def has_dataset(self, dataset_id, group_id=None):
        """
        Evaluates if the dataset exists
        :param dataset_id: The id of the dataset to evaluate
        :param group_id: The optional group id
        :return: True if the dataset exists, False otherwise
        """
        datasets = self.get_datasets(group_id)

        for dataset in datasets:
            if dataset.id == str(dataset_id):
                return True

        return False

    def get_datasets(self, group_id=None):
        """
        Fetches all datasets
        https://msdn.microsoft.com/en-us/library/mt203567.aspx
        :param group_id: The optional group id to get datasets from
        :return: The list of the datasets found
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}'
        # form the headers
        headers = self.client.auth_header

        # get the response
        response = requests.get(url, headers=headers)

        # 200 is the only successful code, raise an exception on any other response code
        if response.status_code != 200:
            raise HTTPError(response, f'Get Datasets request returned http error: {response.status_code}')

        return self.datasets_from_get_datasets_response(response)

    def get_dataset(self, dataset_id, group_id=None):
        """
        Gets a single dataset
        https://msdn.microsoft.com/en-us/library/mt784653.aspx
        :param dataset_id: The id of the dataset to get
        :param group_id: The optional id of the group to get the dataset from
        :return: The dataset returned by the API
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}'
        # form the headers
        headers = self.client.auth_header
        # get the response
        response = requests.get(url, headers=headers)

        # 200 is the only successful code, raise an exception on any other response code
        if response.status_code != 200:
            raise HTTPError(response, f'Get Datasets request returned http error: {response.status_code}')

        return Dataset.from_dict(json.loads(response.text))

    def post_dataset(self, dataset, group_id=None):
        """
        Posts a single dataset
        https://msdn.microsoft.com/en-us/library/mt203562.aspx
        :param dataset: The dataset to push
        :param group_id: The optional group id to push the dataset to
        :return: The pushed dataset as returned by the API
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}'
        # form the headers
        headers = self.client.auth_header
        # form the json dict
        json_dict = DatasetEncoder().default(dataset)

        # get the response
        response = requests.post(url, headers=headers, json=json_dict)

        # 201 - Created. The request was fulfilled and a new Dataset was created.
        if response.status_code != 201:
            raise HTTPError(response, f'Post Datasets request returned http code: {response.status_code}')

        return Dataset.from_dict(json.loads(response.text))

    def delete_dataset(self, dataset_id, group_id=None):
        """
        Deletes a dataset
        :param dataset_id: The id of the dataset to delete
        :param group_id: The optional group id to delete the dataset from
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}'
        # form the headers
        headers = self.client.auth_header

        # get the response
        response = requests.delete(url, headers=headers)

        # 200 is the only successful code
        if response.status_code != 200:
            raise HTTPError(response, f'Delete Dataset request returned http error: {response.status_code}')

    def delete_all_datasets(self, group_id=None):
        """
        Deletes all datasets
        :param group_id: The optional group id of the group to delete all datasets from
        """
        # get all the datasets and delete each one
        datasets = self.get_datasets(group_id)
        for dataset in datasets:
            self.delete_dataset(group_id, dataset.id)

    def get_tables(self, dataset_id, group_id=None):
        """
        Gets tables from a dataset
        https://msdn.microsoft.com/en-us/library/mt203556.aspx
        :param dataset_id: The id of the dataset which to get tables from
        :param group_id: The optional id of the group which to get tables from
        :return: A list of tables from the given group and dataset
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}/{self.tables_snippet}'
        # form the headers
        headers = self.client.auth_header

        # get the response
        response = requests.get(url, headers=headers)

        # 200 is the only successful code, raise an exception on any other response code
        if response.status_code != 200:
            raise HTTPError(response, f'Get Datasets request returned http error: {response.status_code}')

        return self.tables_from_get_tables_response(response)

    def post_rows(self, dataset_id, table_name, rows, group_id=None):
        """
        Posts rows to a table in a given dataset
        https://msdn.microsoft.com/en-us/library/mt203561.aspx
        :param dataset_id: The id of the dataset to post rows to
        :param table_name: The name of the table to post rows to
        :param rows: The rows to post to the table
        :param group_id: The optional id of the group to post rows to
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}/' \
              f'{self.tables_snippet}/{table_name}/{self.rows_snippet}'
        # form the headers
        headers = self.client.auth_header
        # form the json dict
        row_encoder = RowEncoder()
        json_dict = {
            'rows': [row_encoder.default(x) for x in rows]
        }

        # get the response
        response = requests.post(url, headers=headers, json=json_dict)

        # 200 is the only successful code
        if response.status_code != 200:
            raise HTTPError(response, f'Post row request returned http error: {response.status_code}')

    def delete_rows(self, dataset_id, table_name, group_id=None):
        """
        Deletes all rows from a table in a given dataset
        https://msdn.microsoft.com/en-us/library/mt238041.aspx
        :param dataset_id: The id of the dataset to delete the rows from
        :param table_name: The name of the table to delete the rows from
        :param group_id: The optional id of the group to delete the rows from
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}/' \
              f'{self.tables_snippet}/{table_name}/{self.rows_snippet}'
        # form the headers
        headers = self.client.auth_header

        # get the response
        response = requests.delete(url, headers=headers)

        # 200 is the only successful code
        if response.status_code != 200:
            raise HTTPError(response, f'Post row request returned http error: {response.status_code}')

    def get_dataset_parameters(self, dataset_id, group_id=None):
        """
        Gets all parameters for a single dataset
        https://msdn.microsoft.com/en-us/library/mt784653.aspx
        :param dataset_id: The id of the dataset from which you want the parameters
        :param group_id: The optional id of the group to get the dataset's parameters
        :return: The dataset parameters returned by the API
        """
        # group_id can be none, account for it
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # form the url
        url = f'{self.base_url}{groups_part}/{self.datasets_snippet}/{dataset_id}/{self.parameters_snippet}'
        # form the headers
        headers = self.client.auth_header
        # get the response
        response = requests.get(url, headers=headers)

        # 200 is the only successful code, raise an exception on any other response code
        if response.status_code != 200:
            raise HTTPError(response, f'Get Datasets request returned http error: {response.status_code}')

        return json.loads(response.text)

    @classmethod
    def datasets_from_get_datasets_response(cls, response):
        """
        Creates a list of datasets from a http response object
        :param response: The http response object
        :return: A list of datasets created from the given http response object
        """
        # load the response into a dict
        response_dict = json.loads(response.text)
        datasets = []
        # go through entries returned from API
        for entry in response_dict[cls.get_datasets_value_key]:
            datasets.append(Dataset.from_dict(entry))

        return datasets

    @classmethod
    def tables_from_get_tables_response(cls, response):
        """
        Creates a list of tables from a http response object
        :param response: The http response object
        :return: A list of tables created from the given http response object
        """
        # load the response into a dict
        response_dict = json.loads(response.text)
        tables = []
        # go through entries returned from API
        for entry in response_dict[cls.get_datasets_value_key]:
            tables.append(Table.from_dict(entry))

        return tables



# -*- coding: future_fstrings -*-
import requests
import json
import urllib
import re

from requests.exceptions import HTTPError
from .import_class import Import


class Imports:
    # url snippets
    groups_snippet = 'groups'
    imports_snippet = 'imports'
    dataset_displayname_snippet = 'datasetDisplayName'
    nameconflict_snippet = 'nameConflict'

    def __init__(self, client):
        self.client = client
        self.base_url = f'{self.client.api_url}/{self.client.api_version_snippet}/{self.client.api_myorg_snippet}'
        self.upload_file_replace_regex = re.compile('(?![A-z]|[0-9]).')

    @classmethod
    def import_from_response(cls, response):
        response_dict = json.loads(response.text)
        return Import.from_dict(response_dict)

    @classmethod
    def imports_from_response(cls, response):
        response_list = json.loads(response.text).get(Import.value_key)
        return [Import.from_dict(x) for x in response_list]

    def upload_file(self, filename, dataset_displayname, nameconflict=None, group_id=None):
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        # substitute using the regex pattern
        prepared_displayname = re.sub(self.upload_file_replace_regex, '-', dataset_displayname)
        # append the pbix extension (strange yes, but names correctly in powerbi service if so)
        prepared_displayname = f'{prepared_displayname}.pbix'

        url = f'{self.base_url}{groups_part}{self.imports_snippet}' \
              f'?{urllib.parse.urlencode({self.dataset_displayname_snippet : prepared_displayname})}'

        if nameconflict is not None:
            url = url + f'&{self.nameconflict_snippet}={nameconflict}'

        with open(filename, 'rb') as file:
            headers = self.client.auth_header
            response = requests.post(url, headers=headers,
                                     files={
                                         'file': file,
                                     })
        # 200 OK
        if response.status_code == 200:
            import_object = self.import_from_response(response)
        # 202 Accepted
        elif response.status_code == 202:
            import_object = self.import_from_response(response)
        # 490 Conflict (due to name)
        elif response.status_code == 409:
            raise NotImplementedError("Name conflict resolution not implemented yet")
        else:
            raise HTTPError(response, f"Upload file failed with status code: {response.status_code}")

        return import_object

    def get_import(self, import_id, group_id=None):
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        url = f'{self.base_url}{groups_part}{self.imports_snippet}/{import_id}'

        headers = self.client.auth_header
        response = requests.get(url, headers=headers)

        # 200 OK
        if response.status_code == 200:
            import_object = self.import_from_response(response)
        else:
            raise HTTPError(response, f"Get import failed with status code: {response.status_code}")

        return import_object

    def get_imports(self, group_id=None):
        if group_id is None:
            groups_part = '/'
        else:
            groups_part = f'/{self.groups_snippet}/{group_id}/'

        url = f'{self.base_url}{groups_part}{self.imports_snippet}'

        headers = self.client.auth_header
        response = requests.get(url, headers=headers)

        # 200 OK
        if response.status_code == 200:
            import_object = self.imports_from_response(response)
        else:
            raise HTTPError(response, f"Get imports failed with status code: {response.status_code}")

        return import_object



# -*- coding: future_fstrings -*-
import requests
import json

from requests.exceptions import HTTPError
from .group import Group


class Groups:
    # url snippets
    groups_snippet = 'groups'

    # json keys
    get_reports_value_key = 'value'

    def __init__(self, client):
        self.client = client
        self.base_url = f'{self.client.api_url}/{self.client.api_version_snippet}/{self.client.api_myorg_snippet}'

    def count(self):
        """
        Evaluates the number of groups that the client has access to
        :return: int
            The number of groups
        """
        return len(self.get_groups())

    def has_group(self, group_id):
        """
        Evaluates if client has access to the group
        :param group_id:
        :return: bool
            True if the client has access to the group, False otherwise
        """
        groups = self.get_groups()

        for group in groups:
            if group.id == str(group_id):
                return True

        return False

    def get_groups(self):
        """
        Fetches all groups that the client has access to
        :return: list
            The list of groups
        """
        # form the url
        url = f'{self.base_url}/{self.groups_snippet}'
        # form the headers
        headers = self.client.auth_header
        # get the response
        response = requests.get(url, headers=headers)

        # 200 is the only successful code, raise an exception on any other response code
        if response.status_code != 200:
            raise HTTPError(response, f'Get Groups request returned http error: {response.status_code}')

        return self.groups_from_get_groups_response(response)

    @classmethod
    def groups_from_get_groups_response(cls, response):
        """
        Creates a list of groups from a http response object
        :param response:
            The http response object
        :return: list
            The list of groups
        """
        # load the response into a dict
        response_dict = json.loads(response.text)
        groups = []

        # go through entries returned from API
        for entry in response_dict[cls.get_reports_value_key]:
            groups.append(Group.from_dict(entry))

        return groups



# -*- coding: future_fstrings -*-
import json


class Group:
    id_key = 'id'
    name_key = 'name'
    is_readonly_key = 'isReadOnly'
    is_on_dedicated_capacity_key = 'isOnDedicatedCapacity'

    def __init__(self, name, group_id, is_readonly=False, is_on_dedicated_capacity=False):
        self.name = name
        self.id = group_id
        self.is_readonly = is_readonly
        self.is_on_dedicated_capacity = is_on_dedicated_capacity

    @classmethod
    def from_dict(cls, dictionary):
        group_id = dictionary.get(cls.id_key)
        if group_id is None:
            raise RuntimeError("Group dictionary has no id key")

        name = dictionary.get(cls.name_key)
        is_readonly = dictionary.get(cls.is_readonly_key, False)
        is_on_dedicated_capacity = dictionary.get(cls.is_on_dedicated_capacity_key, False)

        return cls(name, group_id, is_readonly, is_on_dedicated_capacity)
